@@ -101,7 +101,7 @@ QT_BEGIN_NAMESPACE
 */
 
 QQuickItemParticle::QQuickItemParticle(QQuickItem *parent) :
-    QQuickParticlePainter(parent), m_fade(true), m_delegate(0)
+    QQuickParticlePainter(parent), m_fade(true), m_lastT(0), m_activeCount(0), m_delegate(0)
 {
     setFlag(QQuickItem::ItemHasContents);
     clock = new Clock(this);
@@ -237,11 +237,12 @@ void QQuickItemParticle::prepareNextFrame()
 
     //TODO: Size, better fade?
     foreach (const QString &str, m_groups){
-        int gIdx = m_system->groupIds[str];
-        int count = m_system->groupData[gIdx]->size();
+        const int gIdx = m_system->groupIds[str];
+        const QVector<QQuickParticleData*> dataVector = m_system->groupData.value(gIdx)->data;
+        const int count = dataVector.size();
 
         for (int i=0; i<count; i++){
-            QQuickParticleData* data = m_system->groupData[gIdx]->data[i];
+            QQuickParticleData* data = dataVector.at(i);
             QQuickItem* item = data->delegate;
             if (!item)
                 continue;
