@@ -48,10 +48,10 @@ QT_BEGIN_NAMESPACE
 class QTextStream;
 class QQmlEnginePrivate;
 
-namespace QQmlJS {
-namespace V4IR {
+namespace QV4 {
+namespace IR {
 
-class LifeTimeInterval {
+class Q_AUTOTEST_EXPORT LifeTimeInterval {
 public:
     struct Range {
         int start;
@@ -88,7 +88,7 @@ public:
 
     void setTemp(const Temp &temp) { this->_temp = temp; }
     Temp temp() const { return _temp; }
-    bool isFP() const { return _temp.type == V4IR::DoubleType; }
+    bool isFP() const { return _temp.type == IR::DoubleType; }
 
     void setFrom(Stmt *from);
     void addRange(int from, int to);
@@ -121,9 +121,23 @@ public:
     void dump(QTextStream &out) const;
     static bool lessThan(const LifeTimeInterval &r1, const LifeTimeInterval &r2);
     static bool lessThanForTemp(const LifeTimeInterval &r1, const LifeTimeInterval &r2);
+
+    void validate() const {
+#if !defined(QT_NO_DEBUG)
+        // Validate the new range
+        if (_end != Invalid) {
+            Q_ASSERT(!_ranges.isEmpty());
+            foreach (const Range &range, _ranges) {
+                Q_ASSERT(range.start >= 0);
+                Q_ASSERT(range.end >= 0);
+                Q_ASSERT(range.start <= range.end);
+            }
+        }
+#endif
+    }
 };
 
-class Optimizer
+class Q_QML_PRIVATE_EXPORT Optimizer
 {
 public:
     Optimizer(Function *function)
@@ -141,7 +155,7 @@ public:
 
     QVector<LifeTimeInterval> lifeRanges() const;
 
-    QSet<V4IR::Jump *> calculateOptionalJumps();
+    QSet<IR::Jump *> calculateOptionalJumps();
 
     static void showMeTheCode(Function *function);
 
@@ -156,11 +170,10 @@ class MoveMapping
     struct Move {
         Expr *from;
         Temp *to;
-        int id;
         bool needsSwap;
 
-        Move(Expr *from, Temp *to, int id)
-            : from(from), to(to), id(id), needsSwap(false)
+        Move(Expr *from, Temp *to)
+            : from(from), to(to), needsSwap(false)
         {}
 
         bool operator==(const Move &other) const
@@ -173,9 +186,9 @@ class MoveMapping
     static Moves sourceUsages(Expr *e, const Moves &moves);
 
 public:
-    void add(Expr *from, Temp *to, int id = 0);
+    void add(Expr *from, Temp *to);
     void order();
-    void insertMoves(BasicBlock *bb, Function *function, bool atEnd) const;
+    QList<IR::Move *> insertMoves(BasicBlock *bb, Function *function, bool atEnd) const;
 
     void dump() const;
 
@@ -185,12 +198,12 @@ private:
                     QList<Move> &swaps) const;
 };
 
-} // V4IR namespace
-} // QQmlJS namespace
+} // IR namespace
+} // QV4 namespace
 
 
-Q_DECLARE_TYPEINFO(QQmlJS::V4IR::LifeTimeInterval, Q_MOVABLE_TYPE);
-Q_DECLARE_TYPEINFO(QQmlJS::V4IR::LifeTimeInterval::Range, Q_PRIMITIVE_TYPE);
+Q_DECLARE_TYPEINFO(QV4::IR::LifeTimeInterval, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QV4::IR::LifeTimeInterval::Range, Q_PRIMITIVE_TYPE);
 
 QT_END_NAMESPACE
 
