@@ -88,7 +88,8 @@ public:
     QPointer<QObject> result;
     QQmlGuardedContextData rootContext;
     QQmlCompiledData *compiledData;
-    QQmlVME vme;
+    QScopedPointer<QQmlObjectCreator> creator;
+    int subComponentToCreate;
     QQmlVMEGuard vmeGuard;
 
     QExplicitlySharedDataPointer<QQmlIncubatorPrivate> waitingOnMe;
@@ -99,8 +100,8 @@ public:
 
     void clear();
 
-    void forceCompletion(QQmlVME::Interrupt &i);
-    void incubate(QQmlVME::Interrupt &i);
+    void forceCompletion(QQmlInstantiationInterrupt &i);
+    void incubate(QQmlInstantiationInterrupt &i);
 };
 
 QT_END_NAMESPACE
