@@ -48,7 +48,7 @@
 #include <private/qqmllocale_p.h>
 #include <private/qv8engine_p.h>
 
-#include <private/qv8profilerservice_p.h>
+#include <private/qv4profilerservice_p.h>
 #include <private/qqmlprofilerservice_p.h>
 #include <private/qqmlglobal_p.h>
 
@@ -72,12 +72,13 @@
 #include <QtCore/qurl.h>
 #include <QtCore/qfile.h>
 #include <QtCore/qcoreapplication.h>
+#include <QtCore/qloggingcategory.h>
 
 QT_BEGIN_NAMESPACE
 
 using namespace QV4;
 
-DEFINE_MANAGED_VTABLE(QtObject);
+DEFINE_OBJECT_VTABLE(QtObject);
 
 struct StaticQtMetaObject : public QObject
 {
@@ -90,7 +91,7 @@ QV4::QtObject::QtObject(ExecutionEngine *v4, QQmlEngine *qmlEngine)
     , m_platform(0)
     , m_application(0)
 {
-    setVTable(&static_vtbl);
+    setVTable(staticVTable());
 
     Scope scope(v4);
     ScopedObject protectThis(scope, this);
@@ -1174,39 +1175,36 @@ ReturnedValue QtObject::method_locale(CallContext *ctx)
     return QQmlLocale::locale(v8engine, code);
 }
 
-namespace {
-
-struct BindingFunction : public QV4::FunctionObject
+QQmlBindingFunction::QQmlBindingFunction(FunctionObject *originalFunction)
+    : QV4::FunctionObject(originalFunction->scope, originalFunction->name())
+    , originalFunction(originalFunction)
 {
-    Q_MANAGED
-    BindingFunction(FunctionObject *originalFunction)
-        : QV4::FunctionObject(originalFunction->scope, originalFunction->name)
-        , originalFunction(originalFunction)
-    {
-        setVTable(&static_vtbl);
-        bindingKeyFlag = true;
-    }
-
-    static ReturnedValue call(Managed *that, CallData *callData)
-    {
-        BindingFunction *This = static_cast<BindingFunction*>(that);
-        return This->originalFunction->call(callData);
-    }
-
-    static void markObjects(Managed *that, ExecutionEngine *e)
-    {
-        BindingFunction *This = static_cast<BindingFunction*>(that);
-        This->originalFunction->mark(e);
-        QV4::FunctionObject::markObjects(that, e);
-    }
-
-    QV4::FunctionObject *originalFunction;
-};
-
-DEFINE_MANAGED_VTABLE(BindingFunction);
-
+    setVTable(staticVTable());
+    bindingKeyFlag = true;
 }
 
+void QQmlBindingFunction::initBindingLocation()
+{
+    QV4::StackFrame frame = engine()->currentStackFrame();
+    bindingLocation.sourceFile = frame.source;
+    bindingLocation.line = frame.line;
+}
+
+ReturnedValue QQmlBindingFunction::call(Managed *that, CallData *callData)
+{
+    QQmlBindingFunction *This = static_cast<QQmlBindingFunction*>(that);
+    return This->originalFunction->call(callData);
+}
+
+void QQmlBindingFunction::markObjects(Managed *that, ExecutionEngine *e)
+{
+    QQmlBindingFunction *This = static_cast<QQmlBindingFunction*>(that);
+    This->originalFunction->mark(e);
+    QV4::FunctionObject::markObjects(that, e);
+}
+
+DEFINE_OBJECT_VTABLE(QQmlBindingFunction);
+
 /*!
     \qmlmethod Qt::binding(function)
 
@@ -1261,7 +1259,7 @@ ReturnedValue QtObject::method_binding(CallContext *ctx)
     if (!f)
         V4THROW_TYPE("binding(): argument (binding expression) must be a function");
 
-    return (new (ctx->engine->memoryManager) BindingFunction(f))->asReturnedValue();
+    return (new (ctx->engine->memoryManager) QQmlBindingFunction(f))->asReturnedValue();
 }
 
 
@@ -1385,19 +1383,24 @@ static QV4::ReturnedValue writeToConsole(ConsoleLogTypes logType, CallContext *c
         result.append(jsStack(v4));
     }
 
+    static QLoggingCategory loggingCategory("qml");
     QV4::StackFrame frame = v4->currentStackFrame();
     const QByteArray baSource = frame.source.toUtf8();
     const QByteArray baFunction = frame.function.toUtf8();
-    QMessageLogger logger(baSource.constData(), frame.line, baFunction.constData());
+    QMessageLogger logger(baSource.constData(), frame.line, baFunction.constData(), loggingCategory.categoryName());
+
     switch (logType) {
     case Log:
-        logger.debug("%s", qPrintable(result));
+        if (loggingCategory.isDebugEnabled())
+            logger.debug("%s", result.toUtf8().constData());
         break;
     case Warn:
-        logger.warning("%s", qPrintable(result));
+        if (loggingCategory.isWarningEnabled())
+            logger.warning("%s", result.toUtf8().constData());
         break;
     case Error:
-        logger.critical("%s", qPrintable(result));
+        if (loggingCategory.isCriticalEnabled())
+            logger.critical("%s", result.toUtf8().constData());
         break;
     default:
         break;
@@ -1422,22 +1425,17 @@ QV4::ReturnedValue ConsoleObject::method_log(CallContext *ctx)
 
 QV4::ReturnedValue ConsoleObject::method_profile(CallContext *ctx)
 {
-    //DeclarativeDebugTrace cannot handle nested profiling
-    //although v8 can handle several profiling at once,
-    //we do not allow that. Hence, we pass an empty(default) title
-    QString title;
     QV4::ExecutionEngine *v4 = ctx->engine;
 
     QV4::StackFrame frame = v4->currentStackFrame();
     const QByteArray baSource = frame.source.toUtf8();
     const QByteArray baFunction = frame.function.toUtf8();
     QMessageLogger logger(baSource.constData(), frame.line, baFunction.constData());
-    if (QQmlProfilerService::startProfiling()) {
-        QV8ProfilerService::instance()->startProfiling(title);
-
-        logger.debug("Profiling started.");
+    if (!QQmlDebugService::isDebuggingEnabled()) {
+        logger.warning("Cannot start profiling because debug service is disabled. Start with -qmljsdebugger=port:XXXXX.");
     } else {
-        logger.warning("Profiling is already in progress. First, end current profiling session.");
+        QQmlProfilerService::instance()->startProfiling(v4->v8Engine->engine());
+        logger.debug("Profiling started.");
     }
 
     return QV4::Encode::undefined();
@@ -1445,11 +1443,6 @@ QV4::ReturnedValue ConsoleObject::method_profile(CallContext *ctx)
 
 QV4::ReturnedValue ConsoleObject::method_profileEnd(CallContext *ctx)
 {
-    //DeclarativeDebugTrace cannot handle nested profiling
-    //although v8 can handle several profiling at once,
-    //we do not allow that. Hence, we pass an empty(default) title
-    QString title;
-
     QV4::ExecutionEngine *v4 = ctx->engine;
 
     QV4::StackFrame frame = v4->currentStackFrame();
@@ -1457,15 +1450,11 @@ QV4::ReturnedValue ConsoleObject::method_profileEnd(CallContext *ctx)
     const QByteArray baFunction = frame.function.toUtf8();
     QMessageLogger logger(baSource.constData(), frame.line, baFunction.constData());
 
-    if (QQmlProfilerService::stopProfiling()) {
-        QV8ProfilerService *profiler = QV8ProfilerService::instance();
-        profiler->stopProfiling(title);
-        QQmlProfilerService::sendProfilingData();
-        profiler->sendProfilingData();
-
-        logger.debug("Profiling ended.");
+    if (!QQmlDebugService::isDebuggingEnabled()) {
+        logger.warning("Ignoring console.profileEnd(): the debug service is disabled.");
     } else {
-        logger.warning("Profiling was not started.");
+        QQmlProfilerService::instance()->stopProfiling(v4->v8Engine->engine());
+        logger.debug("Profiling ended.");
     }
 
     return QV4::Encode::undefined();
