@@ -353,9 +353,9 @@ void QQuickShaderEffectTexture::grab()
             if (m_context->openglContext()->format().samples() <= 1) {
                 m_multisampling = false;
             } else {
-                QList<QByteArray> extensions = QByteArray((const char *)glGetString(GL_EXTENSIONS)).split(' ');
-                m_multisampling = extensions.contains("GL_EXT_framebuffer_multisample")
-                                && extensions.contains("GL_EXT_framebuffer_blit");
+                const QSet<QByteArray> extensions = m_context->openglContext()->extensions();
+                m_multisampling = extensions.contains(QByteArrayLiteral("GL_EXT_framebuffer_multisample"))
+                    && extensions.contains(QByteArrayLiteral("GL_EXT_framebuffer_blit"));
             }
             m_multisamplingChecked = true;
         }
@@ -622,6 +622,8 @@ void QQuickShaderEffectSource::ensureTexture()
     connect(m_texture, SIGNAL(scheduledUpdateCompleted()), this, SIGNAL(scheduledUpdateCompleted()));
 }
 
+static void get_wrap_mode(QQuickShaderEffectSource::WrapMode mode, QSGTexture::WrapMode *hWrap, QSGTexture::WrapMode *vWrap);
+
 QSGTextureProvider *QQuickShaderEffectSource::textureProvider() const
 {
     const QQuickItemPrivate *d = QQuickItemPrivate::get(this);
@@ -634,6 +636,10 @@ QSGTextureProvider *QQuickShaderEffectSource::textureProvider() const
         const_cast<QQuickShaderEffectSource *>(this)->m_provider = new QQuickShaderEffectSourceTextureProvider();
         const_cast<QQuickShaderEffectSource *>(this)->ensureTexture();
         connect(m_texture, SIGNAL(updateRequested()), m_provider, SIGNAL(textureChanged()));
+
+        get_wrap_mode(m_wrapMode, &m_provider->horizontalWrap, &m_provider->verticalWrap);
+        m_provider->mipmapFiltering = mipmap() ? QSGTexture::Linear : QSGTexture::None;
+        m_provider->filtering = smooth() ? QSGTexture::Linear : QSGTexture::Nearest;
         m_provider->sourceTexture = m_texture;
     }
     return m_provider;
