@@ -49,6 +49,7 @@
 #include <private/qqmldebugstatesdelegate_p.h>
 #include <private/qqmlbinding_p.h>
 #include <private/qqmlcontext_p.h>
+#include <private/qquickprofiler_p.h>
 #include <private/qquickapplication_p.h>
 #include <QtQuick/private/qquickpropertychanges_p.h>
 #include <QtQuick/private/qquickstate_p.h>
@@ -189,9 +190,10 @@ void QQmlQtQuick2Module::defineModule()
 
     QQuickValueTypes::registerValueTypes();
 
-    if (QQmlEngineDebugService::isDebuggingEnabled()) {
+    if (QQmlDebugService::isDebuggingEnabled()) {
         QQmlEngineDebugService::instance()->setStatesDelegate(
                     new QQmlQtQuick2DebugStatesDelegate);
+        QQuickProfiler::initialize();
     }
 }
 
