@@ -99,13 +99,8 @@ public:
     QSGContextPrivate()
         : antialiasingMethod(QSGContext::UndecidedAntialiasing)
         , distanceFieldDisabled(qmlDisableDistanceField())
-        , distanceFieldAntialiasing(
-#if !defined(QT_OPENGL_ES) || defined(QT_OPENGL_ES_2_ANGLE)
-              QSGGlyphNode::HighQualitySubPixelAntialiasing
-#else
-              QSGGlyphNode::GrayAntialiasing
-#endif
-              )
+        , distanceFieldAntialiasing(QSGGlyphNode::HighQualitySubPixelAntialiasing)
+        , distanceFieldAntialiasingDecided(false)
     {
     }
 
@@ -117,12 +112,9 @@ public:
     QSGContext::AntialiasingMethod antialiasingMethod;
     bool distanceFieldDisabled;
     QSGDistanceFieldGlyphNode::AntialiasingMode distanceFieldAntialiasing;
-
-    static QOpenGLContext *sharedOpenGLContext;
+    bool distanceFieldAntialiasingDecided;
 };
 
-QOpenGLContext *QSGContextPrivate::sharedOpenGLContext = 0;
-
 class QSGTextureCleanupEvent : public QEvent
 {
 public:
@@ -161,6 +153,8 @@ QSGContext::QSGContext(QObject *parent) :
 {
     Q_D(QSGContext);
     QByteArray mode = qgetenv("QSG_DISTANCEFIELD_ANTIALIASING");
+    if (!mode.isEmpty())
+        d->distanceFieldAntialiasingDecided = true;
     if (mode == "subpixel")
         d->distanceFieldAntialiasing = QSGGlyphNode::HighQualitySubPixelAntialiasing;
     else if (mode == "subpixel-lowq")
@@ -174,18 +168,9 @@ QSGContext::~QSGContext()
 {
 }
 
-/*!
- * This function is used by the Qt WebEngine to set up context sharing
- * across multiple windows. Do not use it for any other purpose.
- */
-void QSGContext::setSharedOpenGLContext(QOpenGLContext *context)
+QSGRenderContext *QSGContext::createRenderContext()
 {
-    QSGContextPrivate::sharedOpenGLContext = context;
-}
-
-QOpenGLContext *QSGContext::sharedOpenGLContext()
-{
-    return QSGContextPrivate::sharedOpenGLContext;
+    return new QSGRenderContext(this);
 }
 
 void QSGContext::renderContextInitialized(QSGRenderContext *renderContext)
@@ -207,13 +192,31 @@ void QSGContext::renderContextInitialized(QSGRenderContext *renderContext)
         }
     }
 
+    // With OpenGL ES, except for Angle on Windows, use GrayAntialiasing, unless
+    // some value had been requested explicitly. This could not be decided
+    // before without a context. Now the context is ready.
+    if (!d->distanceFieldAntialiasingDecided) {
+        d->distanceFieldAntialiasingDecided = true;
+#ifndef Q_OS_WIN
+        if (renderContext->openglContext()->isOpenGLES())
+            d->distanceFieldAntialiasing = QSGGlyphNode::GrayAntialiasing;
+#endif
+    }
+
     static bool dumped = false;
     if (!dumped && qEnvironmentVariableIsSet("QSG_INFO")) {
         dumped = true;
-        qDebug() << "GL_VENDOR:     " << (const char *) glGetString(GL_VENDOR);
-        qDebug() << "GL_RENDERER:   " << (const char *) glGetString(GL_RENDERER);
-        qDebug() << "GL_VERSION:    " << (const char *) glGetString(GL_VERSION);
-        qDebug() << "GL_EXTENSIONS:\n   " << QByteArray((const char *) glGetString(GL_EXTENSIONS)).replace(" ", "\n    ").constData();
+        QSurfaceFormat format = renderContext->openglContext()->format();
+        qDebug() << "R/G/B/A Buffers:   " << format.redBufferSize() << format.greenBufferSize() << format.blueBufferSize() << format.alphaBufferSize();
+        qDebug() << "Depth Buffer:      " << format.depthBufferSize();
+        qDebug() << "Stencil Buffer:    " << format.stencilBufferSize();
+        qDebug() << "Samples:           " << format.samples();
+        qDebug() << "GL_VENDOR:         " << (const char *) glGetString(GL_VENDOR);
+        qDebug() << "GL_RENDERER:       " << (const char *) glGetString(GL_RENDERER);
+        qDebug() << "GL_VERSION:        " << (const char *) glGetString(GL_VERSION);
+        QSet<QByteArray> exts = renderContext->openglContext()->extensions();
+        QByteArray all; foreach (const QByteArray &e, exts) all += ' ' + e;
+        qDebug() << "GL_EXTENSIONS:    " << all.constData();
     }
 
     d->mutex.unlock();
@@ -245,33 +248,15 @@ QSGImageNode *QSGContext::createImageNode()
             : new QSGDefaultImageNode;
 }
 
-/*!
-    Factory function for scene graph backends of the Text elements which supports native
-    text rendering. Used in special cases where native look and feel is a main objective.
-*/
-QSGGlyphNode *QSGContext::createNativeGlyphNode(QSGRenderContext *rc)
-{
-#if defined(QT_OPENGL_ES) && !defined(QT_OPENGL_ES_2_ANGLE)
-    Q_D(QSGContext);
-    if (d->distanceFieldDisabled)
-        return new QSGDefaultGlyphNode;
-    else
-        return createGlyphNode(rc);
-#else
-    Q_UNUSED(rc);
-    return new QSGDefaultGlyphNode;
-#endif
-}
-
 /*!
     Factory function for scene graph backends of the Text elements;
  */
-QSGGlyphNode *QSGContext::createGlyphNode(QSGRenderContext *rc)
+QSGGlyphNode *QSGContext::createGlyphNode(QSGRenderContext *rc, bool preferNativeGlyphNode)
 {
     Q_D(QSGContext);
 
-    if (d->distanceFieldDisabled) {
-        return createNativeGlyphNode(rc);
+    if (d->distanceFieldDisabled || preferNativeGlyphNode) {
+        return new QSGDefaultGlyphNode;
     } else {
         QSGDistanceFieldGlyphNode *node = new QSGDistanceFieldGlyphNode(rc);
         node->setPreferredAntialiasingMode(d->distanceFieldAntialiasing);
@@ -282,8 +267,10 @@ QSGGlyphNode *QSGContext::createGlyphNode(QSGRenderContext *rc)
 QSurfaceFormat QSGContext::defaultSurfaceFormat() const
 {
     QSurfaceFormat format;
-    format.setDepthBufferSize(24);
-    format.setStencilBufferSize(8);
+    static bool useDepth = qEnvironmentVariableIsEmpty("QSG_NO_DEPTH_BUFFER");
+    static bool useStencil = qEnvironmentVariableIsEmpty("QSG_NO_STENCIL_BUFFER");
+    format.setDepthBufferSize(useDepth ? 24 : 0);
+    format.setStencilBufferSize(useStencil ? 8 : 0);
     if (QQuickWindow::hasDefaultAlphaBuffer())
         format.setAlphaBufferSize(8);
     format.setSwapBehavior(QSurfaceFormat::DoubleBuffer);
@@ -349,6 +336,12 @@ QSGRenderContext::~QSGRenderContext()
     invalidate();
 }
 
+void QSGRenderContext::endSync()
+{
+    qDeleteAll(m_texturesToDelete);
+    m_texturesToDelete.clear();
+}
+
 static QBasicMutex qsg_framerender_mutex;
 
 void QSGRenderContext::renderNextFrame(QSGRenderer *renderer, GLuint fboId)
@@ -466,6 +459,9 @@ void QSGRenderContext::invalidate()
     if (!m_gl)
         return;
 
+    qDeleteAll(m_texturesToDelete);
+    m_texturesToDelete.clear();
+
     qDeleteAll(m_textures.values());
     m_textures.clear();
 
@@ -600,7 +596,7 @@ QSGTexture *QSGRenderContext::textureForFactory(QQuickTextureFactory *factory, Q
         m_textures.insert(factory, texture);
         m_mutex.unlock();
 
-        connect(factory, SIGNAL(destroyed(QObject *)), this, SLOT(textureFactoryDestroyed(QObject *)), Qt::DirectConnection);
+        connect(factory, SIGNAL(destroyed(QObject*)), this, SLOT(textureFactoryDestroyed(QObject*)), Qt::DirectConnection);
     }
     return texture;
 }
@@ -608,10 +604,42 @@ QSGTexture *QSGRenderContext::textureForFactory(QQuickTextureFactory *factory, Q
 void QSGRenderContext::textureFactoryDestroyed(QObject *o)
 {
     m_mutex.lock();
-    QSGTexture *t = m_textures.take(static_cast<QQuickTextureFactory *>(o));
+    m_texturesToDelete << m_textures.take(static_cast<QQuickTextureFactory *>(o));
     m_mutex.unlock();
-    if (t)
-        t->deleteLater();
+}
+
+/*!
+    Compile \a shader, optionally using \a vertexCode and \a fragmentCode as
+    replacement for the source code supplied by \a shader.
+
+    If \a vertexCode or \a fragmentCode is supplied, the caller is responsible
+    for setting up attribute bindings.
+
+    \a material is supplied in case the implementation needs to take the
+    material flags into account.
+ */
+
+void QSGRenderContext::compile(QSGMaterialShader *shader, QSGMaterial *material, const char *vertexCode, const char *fragmentCode)
+{
+    Q_UNUSED(material);
+    if (vertexCode || fragmentCode) {
+        Q_ASSERT_X((material->flags() & QSGMaterial::CustomCompileStep) == 0,
+                   "QSGRenderContext::compile()",
+                   "materials with custom compile step cannot have custom vertex/fragment code");
+        QOpenGLShaderProgram *p = shader->program();
+        p->addShaderFromSourceCode(QOpenGLShader::Vertex, vertexCode ? vertexCode : shader->vertexShader());
+        p->addShaderFromSourceCode(QOpenGLShader::Fragment, fragmentCode ? fragmentCode : shader->fragmentShader());
+        p->link();
+        if (!p->isLinked())
+            qWarning() << "shader compilation failed:" << endl << p->log();
+    } else {
+        shader->compile();
+    }
+}
+
+void QSGRenderContext::initialize(QSGMaterialShader *shader)
+{
+    shader->initialize();
 }
 
 QT_END_NAMESPACE
