@@ -58,7 +58,8 @@
 #include "qsgthreadedrenderloop_p.h"
 #include <private/qquickanimatorcontroller_p.h>
 
-#include <private/qqmlprofilerservice_p.h>
+#include <private/qquickprofiler_p.h>
+#include <private/qqmldebugservice_p.h>
 
 /*
    Overall design:
@@ -176,15 +177,13 @@ const QEvent::Type WM_RequestRepaint    = QEvent::Type(QEvent::User + 5);
 // if no windows are rendering.
 const QEvent::Type WM_TryRelease        = QEvent::Type(QEvent::User + 7);
 
-// Passed by the RL to the RL when maybeUpdate is called on the RT to
-// just replay the maybeUpdate later. This typically happens when
-// updatePaintNode() results in a call to QQuickItem::update().
-//const QEvent::Type WM_UpdateLater       = QEvent::Type(QEvent::User + 8); // not used for now
-
 // Passed by the RL to the RT when a QQuickWindow::grabWindow() is
 // called.
 const QEvent::Type WM_Grab              = QEvent::Type(QEvent::User + 9);
 
+// Passed by RL to RT when polish fails and we need to reset the expose sycle.
+const QEvent::Type WM_ResetExposeCycle = QEvent::Type(QEvent::User + 10);
+
 template <typename T> T *windowFor(const QList<T> list, QQuickWindow *window)
 {
     for (int i=0; i<list.size(); ++i) {
@@ -286,10 +285,15 @@ public:
         , pendingUpdate(0)
         , sleeping(false)
         , syncResultedInChanges(false)
+        , exposeCycle(NoExpose)
         , active(false)
         , window(0)
         , stopEventProcessing(false)
     {
+#if defined(Q_OS_QNX) && !defined(Q_OS_BLACKBERRY) && defined(Q_PROCESSOR_X86)
+        // The SDP 6.6.0 x86 MESA driver requires a larger stack than the default.
+        setStackSize(1024 * 1024);
+#endif
         vsyncDelta = qsgrl_animation_interval();
     }
 
@@ -331,6 +335,12 @@ public:
         RepaintRequest      = 0x02
     };
 
+    enum ExposeCycle {
+        NoExpose,
+        ExposePendingSync,
+        ExposePendingSwap
+    };
+
     QSGThreadedRenderLoop *wm;
     QOpenGLContext *gl;
     QSGRenderContext *sgrc;
@@ -340,6 +350,7 @@ public:
     uint pendingUpdate;
     bool sleeping;
     bool syncResultedInChanges;
+    ExposeCycle exposeCycle;
 
     volatile bool active;
 
@@ -366,11 +377,17 @@ bool QSGRenderThread::event(QEvent *e)
         QSG_RT_DEBUG("WM_Expose");
         WMExposeEvent *se = static_cast<WMExposeEvent *>(e);
         Q_ASSERT(!window || window == se->window);
-        pendingUpdate |= RepaintRequest;
         windowSize = se->size;
         window = se->window;
+        Q_ASSERT(exposeCycle == NoExpose);
+        exposeCycle = ExposePendingSync;
         return true; }
 
+    case WM_ResetExposeCycle:
+        QSG_RT_DEBUG("WM_ResetExposeCycle");
+        exposeCycle = NoExpose;
+        return true;
+
     case WM_Obscure: {
         QSG_RT_DEBUG("WM_Obscure");
 
@@ -378,6 +395,7 @@ bool QSGRenderThread::event(QEvent *e)
 
         mutex.lock();
         if (window) {
+            QQuickWindowPrivate::get(window)->fireAboutToStop();
             QSG_RT_DEBUG(" - removed window...");
             window = 0;
         }
@@ -392,6 +410,10 @@ bool QSGRenderThread::event(QEvent *e)
             stopEventProcessing = true;
         if (window)
             pendingUpdate |= SyncRequest;
+        if (exposeCycle == ExposePendingSync) {
+            pendingUpdate |= RepaintRequest;
+            exposeCycle = ExposePendingSwap;
+        }
         return true;
 
     case WM_TryRelease: {
@@ -430,7 +452,7 @@ bool QSGRenderThread::event(QEvent *e)
             QQuickWindowPrivate::get(window)->renderSceneGraph(windowSize);
 
             QSG_RT_DEBUG(" - grabbing result...");
-            *ce->image = qt_gl_read_framebuffer(windowSize, false, false);
+            *ce->image = qt_gl_read_framebuffer(windowSize * window->devicePixelRatio(), false, false);
         }
         QSG_RT_DEBUG(" - waking gui to handle grab result");
         waitCondition.wakeOne();
@@ -516,6 +538,10 @@ void QSGRenderThread::sync()
     if (current) {
         QQuickWindowPrivate *d = QQuickWindowPrivate::get(window);
         bool hadRenderer = d->renderer != 0;
+        // If the scene graph was touched since the last sync() make sure it sends the
+        // changed signal.
+        if (d->renderer)
+            d->renderer->clearChangedFlag();
         d->syncSceneGraph();
         if (!hadRenderer && d->renderer) {
             QSG_RT_DEBUG(" - renderer was created, hooking up changed signal");
@@ -539,7 +565,7 @@ void QSGRenderThread::sync()
 void QSGRenderThread::syncAndRender()
 {
 #ifndef QSG_NO_RENDER_TIMING
-    bool profileFrames = qsg_render_timing || QQmlProfilerService::enabled;
+    bool profileFrames = qsg_render_timing || QQuickProfiler::enabled;
     if (profileFrames) {
         sinceLastTime = threadTimer.nsecsElapsed();
         threadTimer.start();
@@ -600,6 +626,19 @@ void QSGRenderThread::syncAndRender()
 
     QSG_RT_DEBUG(" - rendering done");
 
+    // Though it would be more correct to put this block directly after
+    // fireFrameSwapped in the if (current) branch above, we don't do
+    // that to avoid blocking the GUI thread in the case where it
+    // has started rendering with a bad window, causing makeCurrent to
+    // fail or if the window has a bad size.
+    mutex.lock();
+    if (exposeCycle == ExposePendingSwap) {
+        QSG_RT_DEBUG(" - waking GUI after expose");
+        exposeCycle = NoExpose;
+        waitCondition.wakeOne();
+    }
+    mutex.unlock();
+
 #ifndef QSG_NO_RENDER_TIMING
         if (qsg_render_timing)
             qDebug("Render Thread: window=%p, framedelta=%d, sync=%d, first render=%d, after final swap=%d",
@@ -609,13 +648,10 @@ void QSGRenderThread::syncAndRender()
                    int((renderTime - syncTime)/1000000),
                    int(threadTimer.elapsed() - renderTime/1000000));
 
-        if (QQmlProfilerService::enabled) {
-            QQmlProfilerService::sceneGraphFrame(
-                        QQmlProfilerService::SceneGraphRenderLoopFrame,
-                        syncTime,
-                        renderTime - syncTime,
-                        threadTimer.nsecsElapsed() - renderTime);
-        }
+        Q_QUICK_SG_PROFILE1(QQuickProfiler::SceneGraphRenderLoopFrame, (
+                syncTime,
+                renderTime - syncTime,
+                threadTimer.nsecsElapsed() - renderTime));
 #endif
 }
 
@@ -657,6 +693,8 @@ void QSGRenderThread::run()
     animatorDriver = sgrc->sceneGraphContext()->createAnimationDriver(0);
     animatorDriver->install();
     QUnifiedTimer::instance(true)->setConsistentTiming(QSGRenderLoop::useConsistentTiming());
+    if (QQmlDebugService::isDebuggingEnabled())
+        QQuickProfiler::registerAnimationCallback();
 
     while (active) {
 
@@ -709,7 +747,7 @@ QSGThreadedRenderLoop::QSGThreadedRenderLoop()
 
 QSGRenderContext *QSGThreadedRenderLoop::createRenderContext(QSGContext *sg) const
 {
-    return new QSGRenderContext(sg);
+    return sg->createRenderContext();
 }
 
 void QSGThreadedRenderLoop::maybePostPolishRequest(Window *w)
@@ -816,6 +854,7 @@ void QSGThreadedRenderLoop::show(QQuickWindow *window)
 
     Window win;
     win.window = window;
+    win.actualWindowFormat = window->format();
     win.thread = new QSGRenderThread(this, QQuickWindowPrivate::get(window)->context);
     win.timerId = 0;
     win.updateDuringSync = false;
@@ -911,16 +950,19 @@ void QSGThreadedRenderLoop::handleExposure(Window *w)
 
         if (!w->thread->gl) {
             w->thread->gl = new QOpenGLContext();
-            if (QSGContext::sharedOpenGLContext())
-                w->thread->gl->setShareContext(QSGContext::sharedOpenGLContext());
+            if (QOpenGLContextPrivate::globalShareContext())
+                w->thread->gl->setShareContext(QOpenGLContextPrivate::globalShareContext());
             w->thread->gl->setFormat(w->window->requestedFormat());
             if (!w->thread->gl->create()) {
+                const bool isEs = w->thread->gl->isOpenGLES();
                 delete w->thread->gl;
                 w->thread->gl = 0;
-                qWarning("QtQuick: failed to create OpenGL context");
+                handleContextCreationFailure(w->window, isEs);
                 return;
             }
 
+            QQuickWindowPrivate::get(w->window)->fireOpenGLContextCreated(w->thread->gl);
+
             w->thread->gl->moveToThread(w->thread);
             QSG_GUI_DEBUG(w->window, " - OpenGL context created...");
         }
@@ -941,7 +983,20 @@ void QSGThreadedRenderLoop::handleExposure(Window *w)
     }
 
     w->thread->postEvent(new WMExposeEvent(w->window));
-    polishAndSync(w);
+    bool synced = polishAndSync(w);
+
+    if (synced) {
+        w->thread->mutex.lock();
+        if (w->thread->exposeCycle != QSGRenderThread::NoExpose) {
+            QSG_GUI_DEBUG(w->window, " - waiting for swap to complete...");
+            w->thread->waitCondition.wait(&w->thread->mutex);
+        }
+        Q_ASSERT(w->thread->exposeCycle == QSGRenderThread::NoExpose);
+        w->thread->mutex.unlock();
+    } else {
+        w->thread->postEvent(new QEvent(WM_ResetExposeCycle));
+    }
+    QSG_GUI_DEBUG(w->window, " - handleExposure completed...");
 
     startOrStopAnimationTimer();
 }
@@ -1052,7 +1107,7 @@ void QSGThreadedRenderLoop::releaseResources(QQuickWindow *window, bool inDestru
         if (!window->handle()) {
             QSG_GUI_DEBUG(w->window, " - using fallback surface");
             fallback = new QOffscreenSurface();
-            fallback->setFormat(window->requestedFormat());
+            fallback->setFormat(w->actualWindowFormat);
             fallback->create();
         }
 
@@ -1066,8 +1121,10 @@ void QSGThreadedRenderLoop::releaseResources(QQuickWindow *window, bool inDestru
 }
 
 
-
-void QSGThreadedRenderLoop::polishAndSync(Window *w)
+/* Calls polish on all items, then requests synchronization with the render thread
+ * and blocks until that is complete. Returns false if it aborted; otherwise true.
+ */
+bool QSGThreadedRenderLoop::polishAndSync(Window *w)
 {
     QSG_GUI_DEBUG(w->window, "polishAndSync()");
 
@@ -1075,7 +1132,7 @@ void QSGThreadedRenderLoop::polishAndSync(Window *w)
         QSG_GUI_DEBUG(w->window, " - not exposed, aborting...");
         killTimer(w->timerId);
         w->timerId = 0;
-        return;
+        return false;
     }
 
 
@@ -1084,7 +1141,7 @@ void QSGThreadedRenderLoop::polishAndSync(Window *w)
     qint64 polishTime = 0;
     qint64 waitTime = 0;
     qint64 syncTime = 0;
-    bool profileFrames = qsg_render_timing  || QQmlProfilerService::enabled;
+    bool profileFrames = qsg_render_timing  || QQuickProfiler::enabled;
     if (profileFrames)
         timer.start();
 #endif
@@ -1099,6 +1156,8 @@ void QSGThreadedRenderLoop::polishAndSync(Window *w)
 
     w->updateDuringSync = false;
 
+    emit w->window->afterAnimating();
+
     QSG_GUI_DEBUG(w->window, " - lock for sync...");
     w->thread->mutex.lock();
     m_locked = true;
@@ -1143,15 +1202,14 @@ void QSGThreadedRenderLoop::polishAndSync(Window *w)
                int((syncTime - waitTime)/1000000),
                int((timer.nsecsElapsed() - syncTime)/1000000));
 
-    if (QQmlProfilerService::enabled) {
-        QQmlProfilerService::sceneGraphFrame(
-                    QQmlProfilerService::SceneGraphPolishAndSync,
-                    polishTime,
-                    waitTime - polishTime,
-                    syncTime - waitTime,
-                    timer.nsecsElapsed() - syncTime);
-    }
+    Q_QUICK_SG_PROFILE1(QQuickProfiler::SceneGraphPolishAndSync, (
+            polishTime,
+            waitTime - polishTime,
+            syncTime - waitTime,
+            timer.nsecsElapsed() - syncTime));
 #endif
+
+    return true;
 }
 
 bool QSGThreadedRenderLoop::event(QEvent *e)
