@@ -39,18 +39,16 @@
 **
 ****************************************************************************/
 
-#define GL_GLEXT_PROTOTYPES
-
 #include "qsgtexture_p.h"
 #include <qopenglfunctions.h>
 #include <QtQuick/private/qsgcontext_p.h>
 #include <qthread.h>
-#include <private/qqmlprofilerservice_p.h>
+#include <private/qquickprofiler_p.h>
 #include <private/qqmlglobal_p.h>
 #include <QtGui/qguiapplication.h>
 #include <QtGui/qpa/qplatformnativeinterface.h>
 
-#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
+#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID) && !defined(__UCLIBC__)
 #define CAN_BACKTRACE_EXECINFO
 #endif
 
@@ -83,7 +81,7 @@ static QElapsedTimer qsg_renderer_timer;
 
 QT_BEGIN_NAMESPACE
 
-#if !defined(QT_NO_DEBUG) && defined(QT_OPENGL_ES_2)
+#ifndef QT_NO_DEBUG
 inline static bool isPowerOfTwo(int x)
 {
     // Assumption: x >= 1
@@ -237,6 +235,9 @@ static void qt_debug_remove_texture(QSGTexture* texture)
     the function removedFromAtlas() can be used to extract a
     non-atlassed copy.
 
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
+
     \sa {Scene Graph - Rendering FBOs}, {Scene Graph - Rendering FBOs in a thread}
  */
 
@@ -514,7 +515,7 @@ void QSGTexture::updateBindOptions(bool force)
     }
 
     if (force || d->wrapChanged) {
-#if !defined(QT_NO_DEBUG) && defined(QT_OPENGL_ES_2)
+#ifndef QT_NO_DEBUG
         if (d->horizontalWrap == Repeat || d->verticalWrap == Repeat) {
             bool npotSupported = QOpenGLFunctions(QOpenGLContext::currentContext()).hasOpenGLFeature(QOpenGLFunctions::NPOTTextures);
             QSize size = textureSize();
@@ -533,7 +534,6 @@ QSGPlainTexture::QSGPlainTexture()
     : QSGTexture()
     , m_texture_id(0)
     , m_has_alpha(false)
-    , m_has_mipmaps(false)
     , m_dirty_texture(false)
     , m_dirty_bind_options(false)
     , m_owns_texture(true)
@@ -567,6 +567,7 @@ void QSGPlainTexture::setImage(const QImage &image)
     m_has_alpha = image.hasAlphaChannel();
     m_dirty_texture = true;
     m_dirty_bind_options = true;
+    m_mipmaps_generated = false;
  }
 
 int QSGPlainTexture::textureId() const
@@ -597,18 +598,11 @@ void QSGPlainTexture::setTextureId(int id)
     m_mipmaps_generated = false;
 }
 
-void QSGPlainTexture::setHasMipmaps(bool mm)
-{
-    m_has_mipmaps = mm;
-    m_mipmaps_generated = false;
-}
-
-
 void QSGPlainTexture::bind()
 {
     if (!m_dirty_texture) {
         glBindTexture(GL_TEXTURE_2D, m_texture_id);
-        if (m_has_mipmaps && !m_mipmaps_generated) {
+        if (mipmapFiltering() != QSGTexture::None && !m_mipmaps_generated) {
             QOpenGLContext *ctx = QOpenGLContext::currentContext();
             ctx->functions()->glGenerateMipmap(GL_TEXTURE_2D);
             m_mipmaps_generated = true;
@@ -621,7 +615,7 @@ void QSGPlainTexture::bind()
     m_dirty_texture = false;
 
 #ifndef QSG_NO_RENDER_TIMING
-    bool profileFrames = qsg_render_timing || QQmlProfilerService::enabled;
+    bool profileFrames = qsg_render_timing || QQuickProfiler::enabled;
     if (profileFrames)
         qsg_renderer_timer.start();
 #endif
@@ -636,20 +630,14 @@ void QSGPlainTexture::bind()
                        m_texture_size.width(),
                        m_texture_size.height());
             }
-            if (QQmlProfilerService::enabled) {
-                QQmlProfilerService::sceneGraphFrame(
-                            QQmlProfilerService::SceneGraphTextureDeletion,
-                            qsg_renderer_timer.nsecsElapsed());
-            }
+            Q_QUICK_SG_PROFILE1(QQuickProfiler::SceneGraphTextureDeletion, (
+                    qsg_renderer_timer.nsecsElapsed()));
 #endif
         }
         m_texture_id = 0;
         m_texture_size = QSize();
-        m_has_mipmaps = false;
         m_has_alpha = false;
 
-
-
         return;
     }
 
@@ -687,9 +675,10 @@ void QSGPlainTexture::bind()
 #if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_NO_SDK)
     QString *deviceName =
             static_cast<QString *>(QGuiApplication::platformNativeInterface()->nativeResourceForIntegration("AndroidDeviceName"));
-    static bool wrongfullyReportsBgra8888Support = deviceName->compare(QStringLiteral("samsung SM-T211"), Qt::CaseInsensitive) == 0
-                                                || deviceName->compare(QStringLiteral("samsung SM-T210"), Qt::CaseInsensitive) == 0
-                                                || deviceName->compare(QStringLiteral("samsung SM-T215"), Qt::CaseInsensitive) == 0;
+    static bool wrongfullyReportsBgra8888Support = deviceName != 0
+                                                    && (deviceName->compare(QStringLiteral("samsung SM-T211"), Qt::CaseInsensitive) == 0
+                                                        || deviceName->compare(QStringLiteral("samsung SM-T210"), Qt::CaseInsensitive) == 0
+                                                        || deviceName->compare(QStringLiteral("samsung SM-T215"), Qt::CaseInsensitive) == 0);
 #else
     static bool wrongfullyReportsBgra8888Support = false;
 #endif
@@ -699,7 +688,10 @@ void QSGPlainTexture::bind()
         externalFormat = GL_BGRA;
 #ifdef QT_OPENGL_ES
         internalFormat = GL_BGRA;
-#endif
+#else
+        if (context->isOpenGLES())
+            internalFormat = GL_BGRA;
+#endif // QT_OPENGL_ES
     } else if (!wrongfullyReportsBgra8888Support
                && (context->hasExtension(QByteArrayLiteral("GL_EXT_texture_format_BGRA8888"))
                    || context->hasExtension(QByteArrayLiteral("GL_IMG_texture_format_BGRA8888")))) {
@@ -728,7 +720,7 @@ void QSGPlainTexture::bind()
 #endif
 
 
-    if (m_has_mipmaps) {
+    if (mipmapFiltering() != QSGTexture::None) {
         context->functions()->glGenerateMipmap(GL_TEXTURE_2D);
         m_mipmaps_generated = true;
     }
@@ -751,17 +743,12 @@ void QSGPlainTexture::bind()
 
     }
 
-    if (QQmlProfilerService::enabled) {
-        mipmapTime = qsg_renderer_timer.nsecsElapsed();
-
-        QQmlProfilerService::sceneGraphFrame(
-                    QQmlProfilerService::SceneGraphTexturePrepare,
-                    bindTime,
-                    convertTime - bindTime,
-                    swizzleTime - convertTime,
-                    uploadTime - swizzleTime,
-                    mipmapTime - uploadTime);
-    }
+    Q_QUICK_SG_PROFILE1(QQuickProfiler::SceneGraphTexturePrepare, (
+            bindTime,
+            convertTime - bindTime,
+            swizzleTime - convertTime,
+            uploadTime - swizzleTime,
+            qsg_renderer_timer.nsecsElapsed() - uploadTime));
 
 #endif
 
@@ -783,6 +770,9 @@ void QSGPlainTexture::bind()
 
     To update the content of the texture, call updateTexture() explicitly. Simply calling bind()
     will not update the texture.
+
+    \note All classes with QSG prefix should be used solely on the scene graph's
+    rendering thread. See \l {Scene Graph and Rendering} for more information.
  */
 
 
