@@ -44,6 +44,10 @@
 #include "QtTest/qtestaccessible.h"
 
 #include <QtGui/qaccessible.h>
+#include <QtGui/private/qguiapplication_p.h>
+#include <qpa/qplatformnativeinterface.h>
+#include <qpa/qplatformintegration.h>
+#include <qpa/qplatformaccessibility.h>
 
 #include <QtQuick/qquickview.h>
 #include <QtQuick/qquickitem.h>
@@ -96,6 +100,12 @@ public:
     tst_QQuickAccessible();
     virtual ~tst_QQuickAccessible();
 
+public slots:
+    void initTestCase();
+    void cleanupTestCase();
+    void init();
+    void cleanup();
+
 private slots:
     void commonTests_data();
     void commonTests();
@@ -116,6 +126,37 @@ tst_QQuickAccessible::~tst_QQuickAccessible()
 
 }
 
+void tst_QQuickAccessible::initTestCase()
+{
+    QQmlDataTest::initTestCase();
+    QTestAccessibility::initialize();
+    QPlatformIntegration *pfIntegration = QGuiApplicationPrivate::platformIntegration();
+    pfIntegration->accessibility()->setActive(true);
+}
+
+void tst_QQuickAccessible::cleanupTestCase()
+{
+    QTestAccessibility::cleanup();
+}
+
+void tst_QQuickAccessible::init()
+{
+    QTestAccessibility::clearEvents();
+}
+
+void tst_QQuickAccessible::cleanup()
+{
+    const EventList list = QTestAccessibility::events();
+    if (!list.isEmpty()) {
+        qWarning("%d accessibility event(s) were not handled in testfunction '%s':", list.count(),
+                 QString(QTest::currentTestFunction()).toLatin1().constData());
+        for (int i = 0; i < list.count(); ++i)
+            qWarning(" %d: Object: %p Event: '%s' Child: %d", i + 1, list.at(i)->object(),
+                     qAccessibleEventString(list.at(i)->type()), list.at(i)->child());
+    }
+    QTestAccessibility::clearEvents();
+}
+
 void tst_QQuickAccessible::commonTests_data()
 {
     QTest::addColumn<QString>("accessibleRoleFileName");
@@ -141,50 +182,7 @@ void tst_QQuickAccessible::commonTests()
     QVERIFY(iface);
 
     delete view;
-}
-
-
-
-QString eventName(const int ev)
-{
-    switch (ev) {
-    case 0x0001: return "SoundPlayed";
-    case 0x0002: return "Alert";
-    case 0x0003: return "ForegroundChanged";
-    case 0x0004: return "MenuStart";
-    case 0x0005: return "MenuEnd";
-    case 0x0006: return "PopupMenuStart";
-    case 0x0007: return "PopupMenuEnd";
-    case 0x000C: return "ContextHelpStart";
-    case 0x000D: return "ContextHelpEnd";
-    case 0x000E: return "DragDropStart";
-    case 0x000F: return "DragDropEnd";
-    case 0x0010: return "DialogStart";
-    case 0x0011: return "DialogEnd";
-    case 0x0012: return "ScrollingStart";
-    case 0x0013: return "ScrollingEnd";
-    case 0x0018: return "MenuCommand";
-    case 0x8000: return "ObjectCreated";
-    case 0x8001: return "ObjectDestroyed";
-    case 0x8002: return "ObjectShow";
-    case 0x8003: return "ObjectHide";
-    case 0x8004: return "ObjectReorder";
-    case 0x8005: return "Focus";
-    case 0x8006: return "Selection";
-    case 0x8007: return "SelectionAdd";
-    case 0x8008: return "SelectionRemove";
-    case 0x8009: return "SelectionWithin";
-    case 0x800A: return "StateChanged";
-    case 0x800B: return "LocationChanged";
-    case 0x800C: return "NameChanged";
-    case 0x800D: return "DescriptionChanged";
-    case 0x800E: return "ValueChanged";
-    case 0x800F: return "ParentChanged";
-    case 0x80A0: return "HelpChanged";
-    case 0x80B0: return "DefaultActionChanged";
-    case 0x80C0: return "AcceleratorChanged";
-    default: return "Unknown Event";
-    }
+    QTestAccessibility::clearEvents();
 }
 
 void tst_QQuickAccessible::quickAttachedProperties()
@@ -259,6 +257,7 @@ void tst_QQuickAccessible::quickAttachedProperties()
         }
         delete object;
     }
+    QTestAccessibility::clearEvents();
 }
 
 
@@ -309,6 +308,7 @@ void tst_QQuickAccessible::basicPropertiesTest()
     QCOMPARE(text2->indexOfChild(item), -1);
 
     delete window;
+    QTestAccessibility::clearEvents();
 }
 
 QAccessibleInterface *topLevelChildAt(QAccessibleInterface *iface, int x, int y)
@@ -374,15 +374,27 @@ void tst_QQuickAccessible::hitTest()
     QCOMPARE(rootItemIface->text(QAccessible::Name), QLatin1String("rect201"));
 
     delete window;
+    QTestAccessibility::clearEvents();
 }
 
 void tst_QQuickAccessible::checkableTest()
 {
-    QQuickView *window = new QQuickView();
+    QScopedPointer<QQuickView> window(new QQuickView());
     window->setSource(testFileUrl("checkbuttons.qml"));
     window->show();
+    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window.data()));
 
-    QAccessibleInterface *iface = QAccessible::queryAccessibleInterface(window);
+    QQuickItem *contentItem = window->contentItem();
+    QVERIFY(contentItem);
+    QQuickItem *rootItem = contentItem->childItems().first();
+    QVERIFY(rootItem);
+
+    // the window becomes active
+    QAccessible::State activatedChange;
+    activatedChange.active = true;
+
+    QAccessibleInterface *iface = QAccessible::queryAccessibleInterface(window.data());
     QVERIFY(iface);
     QAccessibleInterface *root = iface->child(0);
 
@@ -391,9 +403,28 @@ void tst_QQuickAccessible::checkableTest()
     QVERIFY(!(button1->state().checked));
     QVERIFY(!(button1->state().checkable));
 
+    QVERIFY(button1->state().focusable);
+    QVERIFY(!button1->state().focused);
+
+    QTestAccessibility::clearEvents();
+
+    // set properties
+    QQuickItem *button1item = qobject_cast<QQuickItem*>(rootItem->childItems().at(0));
+    QVERIFY(button1item);
+    QCOMPARE(button1item->objectName(), QLatin1String("button1"));
+    button1item->forceActiveFocus();
+    QVERIFY(button1->state().focusable);
+    QVERIFY(button1->state().focused);
+
+    QAccessibleEvent focusEvent(button1item, QAccessible::Focus);
+    QVERIFY_EVENT(&focusEvent);
+
     QAccessibleInterface *button2 = root->child(1);
     QVERIFY(!(button2->state().checked));
     QVERIFY(button2->state().checkable);
+    QQuickItem *button2item = qobject_cast<QQuickItem*>(rootItem->childItems().at(1));
+    QVERIFY(button2item);
+    QCOMPARE(button2item->objectName(), QLatin1String("button2"));
 
     QAccessibleInterface *button3 = root->child(2);
     QVERIFY(button3->state().checked);
@@ -401,12 +432,28 @@ void tst_QQuickAccessible::checkableTest()
 
     QAccessibleInterface *checkBox1 = root->child(3);
     QCOMPARE(checkBox1->role(), QAccessible::CheckBox);
-    QVERIFY((checkBox1->state().checked));
+    QVERIFY(checkBox1->state().checked);
     QVERIFY(checkBox1->state().checkable);
+    QQuickItem *checkbox1item = qobject_cast<QQuickItem*>(rootItem->childItems().at(3));
+    QVERIFY(checkbox1item);
+    QCOMPARE(checkbox1item->objectName(), QLatin1String("checkbox1"));
+
+    checkbox1item->setProperty("checked", false);
+    QVERIFY(!checkBox1->state().checked);
+    QAccessible::State checkState;
+    checkState.checked = true;
+    QAccessibleStateChangeEvent checkChanged(checkbox1item, checkState);
+    QVERIFY_EVENT(&checkChanged);
+
+    checkbox1item->setProperty("checked", true);
+    QVERIFY(checkBox1->state().checked);
+    QVERIFY_EVENT(&checkChanged);
 
     QAccessibleInterface *checkBox2 = root->child(4);
     QVERIFY(!(checkBox2->state().checked));
     QVERIFY(checkBox2->state().checkable);
+
+    QTestAccessibility::clearEvents();
 }
 
 QTEST_MAIN(tst_QQuickAccessible)
