@@ -97,6 +97,7 @@ private slots:
     void stopAtBounds();
     void stopAtBounds_data();
     void nestedMouseAreaUsingTouch();
+    void pressDelayWithLoader();
 
 private:
     void flickWithTouch(QWindow *window, QTouchDevice *touchDevice, const QPoint &from, const QPoint &to);
@@ -1440,17 +1441,23 @@ void tst_qquickflickable::stopAtBounds_data()
 {
     QTest::addColumn<bool>("transpose");
     QTest::addColumn<bool>("invert");
+    QTest::addColumn<bool>("pixelAligned");
 
-    QTest::newRow("left") << false << false;
-    QTest::newRow("right") << false << true;
-    QTest::newRow("top") << true << false;
-    QTest::newRow("bottom") << true << true;
+    QTest::newRow("left") << false << false << false;
+    QTest::newRow("right") << false << true << false;
+    QTest::newRow("top") << true << false << false;
+    QTest::newRow("bottom") << true << true << false;
+    QTest::newRow("left,pixelAligned") << false << false << true;
+    QTest::newRow("right,pixelAligned") << false << true << true;
+    QTest::newRow("top,pixelAligned") << true << false << true;
+    QTest::newRow("bottom,pixelAligned") << true << true << true;
 }
 
 void tst_qquickflickable::stopAtBounds()
 {
     QFETCH(bool, transpose);
     QFETCH(bool, invert);
+    QFETCH(bool, pixelAligned);
 
     QQuickView view;
     view.setSource(testFileUrl("stopAtBounds.qml"));
@@ -1469,6 +1476,7 @@ void tst_qquickflickable::stopAtBounds()
         flickable->setContentY(invert ? 100 : 0);
     else
         flickable->setContentX(invert ? 100 : 0);
+    flickable->setPixelAligned(pixelAligned);
 
     const int threshold = qApp->styleHints()->startDragDistance();
 
@@ -1518,6 +1526,29 @@ void tst_qquickflickable::stopAtBounds()
     }
 
     QTest::mouseRelease(&view, Qt::LeftButton, 0, position);
+
+    if (transpose) {
+        flickable->setContentY(invert ? 100 : 0);
+    } else {
+        flickable->setContentX(invert ? 100 : 0);
+    }
+    if (invert)
+        flick(&view, QPoint(20,20), QPoint(120,120), 100);
+    else
+        flick(&view, QPoint(120,120), QPoint(20,20), 100);
+
+    QVERIFY(flickable->isFlicking());
+    if (transpose) {
+        if (invert)
+            QTRY_COMPARE(flickable->isAtYBeginning(), true);
+        else
+            QTRY_COMPARE(flickable->isAtYEnd(), true);
+    } else {
+        if (invert)
+            QTRY_COMPARE(flickable->isAtXBeginning(), true);
+        else
+            QTRY_COMPARE(flickable->isAtXEnd(), true);
+    }
 }
 
 void tst_qquickflickable::nestedMouseAreaUsingTouch()
@@ -1550,6 +1581,23 @@ void tst_qquickflickable::nestedMouseAreaUsingTouch()
     QVERIFY(nested->y() < 100.0);
 }
 
+// QTBUG-31328
+void tst_qquickflickable::pressDelayWithLoader()
+{
+    QScopedPointer<QQuickView> window(new QQuickView);
+    window->setSource(testFileUrl("pressDelayWithLoader.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
+    QVERIFY(window->rootObject() != 0);
+
+    // do not crash
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(150, 150));
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(150, 150));
+}
+
 QTEST_MAIN(tst_qquickflickable)
 
 #include "tst_qquickflickable.moc"
